let data;

const xLabel = "Total Generation (TWh)";
const yLabel = "Share of production (%)";
const boxWidth = 30;
const NUM_YRS = 21;
const NUM_AREAS = 10;

const fetchCSV = async() => {
    //fetch csv data
    const pathToCSV = "https://gist.githubusercontent.com/colmpat/43366b017598cc4d49a6bbe0404c743a/raw/fbd18baebd690ed9d8fb6ee7435d27bc26abd5bf/Europe-Power-Sector-2020-Heatmap.csv";
    
    data = await d3.csv(pathToCSV, d => {
        return {
            year: parseInt(d["Year"]),
            area: d["Area"],
            share: parseFloat(d[yLabel])
        };
    });
    
}

const drawHeatmap = async() => {

    //load in csv
    try {
        await fetchCSV();
    } catch (e) {
        console.log("there was an error loading the csv file:\n", e);
    }

    let canvasWidth = 800;
    let canvasHeight = 420;
    let margin = 25;
    let w = (boxWidth + 2) * NUM_YRS;
    let h = (boxWidth + 2) * NUM_AREAS;

    let svg = d3.select("body").append("svg")
        .attr("width", canvasWidth)
        .attr("height", canvasHeight);

    //------axis scales-------------
    let xScale = d3.scaleBand()
        .domain(data.map(d => d.year))
        .range([0, w]);
    let xAxis = d3.axisBottom(xScale);

    let yScale = d3.scaleBand()
        .domain(data.map(d => d.area))
        .range([h, 0]);
    let yAxis = d3.axisLeft(yScale)
        .tickFormat(d => d);

    //color scale
    let colorScale = d3.interpolateRgb.gamma(2)("rgb(18,22,25)", "rgb(110,235,131)");

    //------Title-----------
    svg.append("text")
        .attr("text-anchor", "middle")
        .attr("x", canvasWidth / 2)
        .attr("y", 30)
        .text("Average Share of Production for Renewables per Country, from 2000 to 2020");

    //prepare plot container
    let container_g = svg.append("g")
        .attr("transform", `translate(${canvasWidth - (w + margin)}, ${(canvasHeight - h) / 2})`);

    //draw x and y axis
    container_g.append("g")
        .attr("transform", `translate(0, ${h})`)
        .call(xAxis);
    container_g.append("g")
        .call(yAxis);

    //prepare wrapper groups:
    let groups = container_g.selectAll(null)
        .data(data)
        .enter()
        .append("g")
        .attr("class", "box-wrapper");

    //draw boxes
    groups.append("rect")
        .attr("x", d => xScale(d.year) + 3)
        .attr("y", d => yScale(d.area))
        .attr("class", "box")
        .attr("width", 30)
        .attr("height", 30)
        .attr("fill", d => colorScale(d.share / 100))

    //add text for hover interaction to the groups
    groups.append("text")
        .attr("class", "box label")
        .attr("text-anchor", "left")
        .attr("x", d => xScale(d.year) + 5)
        .attr("y", d => yScale(d.area) + boxWidth / 2)
        .style('fill', d => d.share < 50 ? '#F4F4F9' : 'rgb(18,22,25)')
        .text(d => d.share + "%");

}

drawHeatmap();
